@@ -40,7 +40,7 @@ alpm_transflag_t trans_flags = 0;
 
 alpm_list_t *spec = NULL, *add = NULL, *rem = NULL, *files = NULL;
 alpm_list_t **list = &spec;
-alpm_list_t *ignore_pkg = NULL, *ignore_group = NULL;
+alpm_list_t *ignore_pkg = NULL, *ignore_group = NULL, *assume_installed = NULL;
 int printonly = 0, noconfirm = 0, sysupgrade = 0, downgrade = 0, dbsync = 0;
 int nohooks = 0, isep = '\n';
 int resolve_conflict = 0, resolve_replacement = 0;
@@ -50,6 +50,7 @@ const char *dbext = NULL, *sysroot = NULL;
 
 enum longopt_flags {
   FLAG_ADD = 1000,
+  FLAG_ASSUME_INSTALLED,
   FLAG_ASDEPS,
   FLAG_ASEXPLICIT,
   FLAG_CACHEDIR,
@@ -156,6 +157,8 @@ void usage(int ret) {
   hputs("   --no-confirm       assume default responses to all prompts");
   hputs("   --no-deps          ignore dependency version restrictions");
   hputs("                      (pass twice to ignore dependencies altogether)");
+  hputs("   --assume-installed=<package[=version]>");
+  hputs("                      behave as if <package> is installed");
   hputs("   --no-hooks         do not run transaction hooks");
   hputs("   --no-scriptlet     do not run package install scripts");
   hputs("   --null=[sep]       parse stdin as <sep> separated values (default NUL)");
@@ -259,6 +262,7 @@ pu_config_t *parse_opts(int argc, char **argv) {
     { "file", no_argument, NULL, FLAG_FILE         },
     { "remove", no_argument, NULL, FLAG_REMOVE       },
 
+    { "assume-installed", required_argument, NULL, FLAG_ASSUME_INSTALLED     },
     { "cache-dir", required_argument, NULL, FLAG_CACHEDIR     },
     { "cachedir", required_argument, NULL, FLAG_CACHEDIR     },
     { "config", required_argument, NULL, FLAG_CONFIG       },
@@ -351,6 +355,9 @@ pu_config_t *parse_opts(int argc, char **argv) {
         break;
 
       /* general options */
+      case FLAG_ASSUME_INSTALLED:
+        alpm_list_append_strdup(&assume_installed, optarg);
+        break;
       case FLAG_CACHEDIR:
         FREELIST(config->cachedirs);
         config->cachedirs = alpm_list_add(NULL, strdup(optarg));
@@ -663,9 +670,8 @@ void print_q_resolution(alpm_question_t *question) {
     case ALPM_QUESTION_CONFLICT_PKG: {
       alpm_question_conflict_t *q = (alpm_question_conflict_t *) question;
       alpm_conflict_t *c = q->conflict;
-      alpm_list_t *localpkgs = alpm_db_get_pkgcache(alpm_get_localdb(handle));
-      alpm_pkg_t *newpkg = alpm_pkg_find(alpm_trans_get_add(handle), c->package1);
-      alpm_pkg_t *oldpkg = alpm_pkg_find(localpkgs, c->package2);
+      alpm_pkg_t *newpkg = c->package1;
+      alpm_pkg_t *oldpkg = c->package2;
 
       if (q->remove) {
         pu_ui_notice("uninstalling package '%s-%s' due to conflict with '%s-%s'",
@@ -716,18 +722,9 @@ void print_q_resolution(alpm_question_t *question) {
     case ALPM_QUESTION_IMPORT_KEY: {
       alpm_question_import_key_t *q = &question->import_key;
       if (q->import) {
-        alpm_pgpkey_t *key = q->key;
-        char created[16];
-        time_t time = (time_t) key->created;
-
-        if (strftime(created, 12, "%Y-%m-%d", localtime(&time)) == 0) {
-          strcpy(created, "(unknown)");
-        }
-
-        pu_ui_notice((key->revoked
-                ? "importing PGP key %u%c/%s '%s', created: %s (revoked)"
-                : "importing PGP key %u%c/%s '%s', created: %s"),
-            key->length, key->pubkey_algo, key->fingerprint, key->uid, created);
+        q->uid
+          ? pu_ui_notice( "importing PGP key %s (%s)", q->fingerprint, q->uid)
+          : pu_ui_notice( "importing PGP key %s", q->fingerprint);
       }
     }
     break;
@@ -786,9 +783,8 @@ void cb_question(void *ctx, alpm_question_t *question) {
       if ((autoset = (resolve_conflict != RESOLVE_CONFLICT_PROMPT))) {
         alpm_question_conflict_t *q = (alpm_question_conflict_t *) question;
         alpm_conflict_t *c = q->conflict;
-        alpm_list_t *localpkgs = alpm_db_get_pkgcache(alpm_get_localdb(handle));
-        alpm_pkg_t *newpkg = alpm_pkg_find(alpm_trans_get_add(handle), c->package1);
-        alpm_pkg_t *oldpkg = alpm_pkg_find(localpkgs, c->package2);
+        alpm_pkg_t *newpkg = c->package1;
+        alpm_pkg_t *oldpkg = c->package2;
 
         q->remove = should_remove_conflict(resolve_conflict, newpkg, oldpkg);
       }
@@ -871,6 +867,16 @@ int main(int argc, char **argv) {
   alpm_option_set_dlcb(handle, pu_ui_cb_download, NULL);
   alpm_option_set_logcb(handle, cb_log, NULL);
 
+  for (i = assume_installed; i; i = i->next) {
+    alpm_depend_t *d = alpm_dep_from_string(i->data);
+    if(!d) {
+      pu_ui_error("unable to parse dependency string '%s'", i->data);
+      ret = 1;
+      goto cleanup;
+    }
+    alpm_option_add_assumeinstalled(handle, d);
+    alpm_dep_free(d);
+  }
   for (i = ignore_pkg; i; i = i->next) {
     alpm_option_add_ignorepkg(handle, i->data);
   }
@@ -996,7 +1002,8 @@ int main(int argc, char **argv) {
         for (i = err_data; i; i = alpm_list_next(i)) {
           alpm_conflict_t *conflict = i->data;
           fprintf(stderr, "error: package conflict (%s %s)\n",
-              conflict->package1, conflict->package2);
+              alpm_pkg_get_name(conflict->package1),
+              alpm_pkg_get_name(conflict->package2));
           alpm_conflict_free(conflict);
         }
         break;
@@ -1071,5 +1078,3 @@ cleanup:
 
   return ret;
 }
-
-/* vim: set ts=2 sw=2 noet: */
